'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var mapWindowEventNameToListeners = {};
var mapWindowEventNameToSharedListener = {};

exports.default = function () {
  var element = arguments.length <= 0 || arguments[0] === undefined ? window : arguments[0];


  // If the `element` is `window`, persist `mapEventNameToListeners` and
  // `mapEventNameToSharedListener` across different calls
  // to `sharedEventListeners`.
  var isWindow = element === window;

  // Maps `eventName` to the list of listeners associated with each
  // respective `eventName`.
  var mapEventNameToListeners = isWindow ? mapWindowEventNameToListeners : {};

  // Maps `eventName` to the shared listener bound to `element`.
  var mapEventNameToSharedListener = isWindow ? mapWindowEventNameToSharedListener : {};

  // Remove the `listener` from the list of listeners associated with
  // the `eventName`.
  function remove(eventName, listener) {
    var listeners = mapEventNameToListeners[eventName];
    var listenerIndex = listeners ? listeners.indexOf(listener) : -1;
    if (listenerIndex === -1) {
      throw new Error('The listener to be removed does not exist');
    }
    listeners.splice(listenerIndex, 1);
    if (listeners.length === 0) {
      delete mapEventNameToListeners[eventName];
      delete mapEventNameToSharedListener[eventName];
      element.removeEventListener(eventName, mapEventNameToSharedListener[eventName]);
    }
  }

  // Add the `listener` to the list of listeners associated with
  // the `eventName`.
  function add(eventName, listener) {
    var listeners = mapEventNameToListeners[eventName];
    if (!listeners) {
      var sharedListener = function sharedListener() {
        listeners.forEach(function (listener) {
          listener();
        });
      };

      listeners = [listener];
      mapEventNameToListeners[eventName] = listeners;

      mapEventNameToSharedListener[eventName] = sharedListener;
      element.addEventListener(eventName, sharedListener);
    } else {
      mapEventNameToListeners[eventName].push(listener);
    }
    return remove.bind(null, eventName, listener);
  }

  // Expose public API.
  return {
    add: add,
    remove: remove
  };
};